module BinTrees where
  
import Codec.Picture
import ShapeGraphics


data BinaryTree a
  = Leaf
  | Node a (BinaryTree a) (BinaryTree a)
  deriving Show

insertTree :: Ord a => a -> BinaryTree a -> BinaryTree a
insertTree x Leaf
  = Node x Leaf Leaf
insertTree newValue (Node nodeValue leftSubtree rightSubtree)
  | newValue < nodeValue = Node nodeValue (insertTree newValue leftSubtree) rightSubtree
  | otherwise            = Node nodeValue leftSubtree (insertTree newValue rightSubtree)


-- Render a binary tree as a 'ShapeGraphics' picture.
--
renderTree :: Show a => BinaryTree a -> Picture
renderTree tree = movePicture (Vector ((1000 - w)/2) ((1000 - h)/2)) p1
  where 
    dist     = 40
    nodeSize = 50
    height   = 100
    (p1, x, h, w) = renderTree' tree
    renderTree' Leaf 
      = (leafPoly, nodeSize/2, nodeSize, nodeSize)         
    renderTree' (Node x t1 t2) 
      = ([Path [Point (width1   -5) 50, Point t1x 100]  black Solid] ++ 
         [Path [Point (width1 + 45) 50, Point (width1 + t2x + 40) 100] black Solid] ++ 
          movePicture (Vector (width1 + dist/2) 0) (nodePic x) ++
          movePicture (Vector 0                 height) t1Pic  ++
          movePicture (Vector (width1 + dist)   height) t2Pic,
         width1 + dist/2, 
         height + max height1 height2, 
         width1 + width2 + dist) 
      where
        (t1Pic, t1x, height1, width1) = renderTree' t1
        (t2Pic, t2x, height2, width2) = renderTree' t2
    nodePic x  
      = let
          size = fromIntegral $ length $ show x
        in [TextBox (Point (- size * 10) 40) black (show x) (nodeSize/2), 
            Polygon [Point (- nodeSize/2) 0, Point (-nodeSize/2) nodeSize, 
                     Point (nodeSize/2) nodeSize, Point (nodeSize/2) 0, 
                     Point 0 0] 
                    black Solid NoFill]
    leafPoly :: Picture
    leafPoly = [Polygon [Point 0 0, Point 0 nodeSize, Point nodeSize nodeSize, 
                         Point nodeSize 0, Point 0 0, Point nodeSize nodeSize, 
                         Point 0 nodeSize, Point nodeSize 0] black Solid NoFill]
